// ****************************************************************
// Sketch Esp8266 Zeitschaltuhr Dual Modular(Tab)
// created: Jens Fleischer, 2019-03-08
// last mod: Jens Fleischer, 2020-12-21
// For more information visit: https://fipsok.de
// ****************************************************************
// Hardware: Esp8266, Relais Modul o. Mosfet IRF3708 o. Fotek SSR-40 DA
// für Relais Modul
// GND an GND
// IN1 an D5 = GPIO14
// IN2 an D6 = GPIO12
// VCC an VIN -> je nach verwendeten Esp.. möglich
// Jumper JD-VCC VCC
// alternativ ext. 5V Netzteil verwenden
//
// für Mosfet IRF3708
// Source an GND
// Mosfet1 Gate an D5 = GPIO14
// Mosfet2 Gate an D6 = GPIO12
//
// für 3V Solid State Relais
// GND an GND
// SSR1 Input + an D5 = GPIO14
// SSR2 Input + an D6 = GPIO12
//
// Software: Esp8266 Arduino Core 2.4.2 / 2.5.2 / 2.6.3 / 2.7.4
// Getestet auf: Nodemcu, Wemos D1 Mini Pro
/******************************************************************
  Copyright (c) 2018 Jens Fleischer. All rights reserved.

  This file is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.
  This file is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.
*******************************************************************/
// Diese Version von Wochenzeitschaltuhr sollte als Tab eingebunden werden.
// #include <LittleFS.h> #include <ESP8266WebServer.h> müssen im Haupttab aufgerufen werden.
// Die Funktionalität des ESP8266 Webservers ist erforderlich.
// Der Lokalzeit Tab ist zum ausführen der Zeitschaltuhr einzubinden.
// Die Funktion "setupSchaltUhr();" muss im Setup aufgerufen werden.
// Zum schalten muss die Funktion "dualSchaltuhr();" im loop(); aufgerufen werden.
/**************************************************************************************/

constexpr auto ACTIVE = HIGH;                    // LOW für LOW aktive Relais oder HIGH für HIGH aktive (zB. SSR, Mosfet) einstellen
constexpr auto COUNT = 12;                      // Anzahl Schaltzeiten (analog Html Dokument) einstellen 2 bis 40
char switchTime[COUNT * 2][6];
byte switchActive[COUNT];
byte wday[COUNT];
bool relState[] = {!ACTIVE, !ACTIVE};

void setupTimerSwitch() {
  File file = LittleFS.open(existFolder("Config") + "/dtime.dat", "r");
  if (file) {                                                          // Einlesen aller Daten falls die Datei im LittleFS vorhanden ist.
    file.read(switchActive, sizeof(switchActive));
    file.read(wday, sizeof(wday));
    while (file.read() != '\n');
    for (auto i = 0; i < COUNT * 2; i++) {
      file.readBytesUntil('\n', switchTime[i], sizeof(switchTime[i]));
    }
    file.close();
  } else {                                                             // Sollte die Datei nicht existieren
    for (auto i = 0; i < COUNT; i++) {
      switchActive[i] = 1;                                             // werden alle Schaltzeiten
      wday[i] = ~wday[i];                                              // und alle Wochentage aktiviert.
    }
  }
  server.on("/timer", HTTP_POST, []() {
    if (server.args() == 1) {
      switchActive[server.argName(0).toInt()] = server.arg(0).toInt();
      toSave();
      String temp = "\"";
      for (auto i = 0; i < COUNT; i++) {
        temp += switchActive[i];
      }
      temp += "\"";
      server.send(200, "application/json", temp);
    }
    if (server.hasArg("sTime")) {
      char str[COUNT * 14];
      strcpy (str, server.arg("sTime").c_str());
      char* ptr = strtok(str, ",");
      byte i = 0, j = 0;
      while (ptr != NULL) {
        strcpy (switchTime[i++], ptr);
        ptr = strtok(NULL, ",");
      }
      if (server.arg("sDay")) {
        i = 0;
        strcpy (str, server.arg("sDay").c_str());
        char* ptr = strtok(str, ",");
        while (ptr != NULL) {
          wday[i++] = atoi(ptr);
          ptr = strtok(NULL, ",");
        }
        toSave();
      }
      else {
        server.send(400, "");
      }
    }
    String temp = "[";
    for (auto i = 0; i < COUNT * 2; i++) {
      if (temp != "[") temp += ',';
      temp += (String)"\"" + switchTime[i] + "\"";
    }
    temp += ",\"";
    for (auto i = 0; i < COUNT; i++) {
      temp += switchActive[i];
    }
    for (auto& elem : wday) {
      temp += "\",\"";
      temp += elem;
    }
    temp += "\"]";
    server.send(200, "application/json", temp);
  });
  server.on("/timer", HTTP_GET, []() {
    if (server.hasArg("tog") && server.arg(0) == "0") {
      relState[0] = !relState[0];                                      // Relais1 Status manuell ändern
      timeDataLogger(0, server.client().remoteIP().toString());        // Funktionsaufruf Zeitdatenlogger
    }
    if (server.hasArg("tog") && server.arg(0) == "1") {
      relState[1] = !relState[1];                                      // Relais2 Status manuell ändern
      timeDataLogger(1, server.client().remoteIP().toString());        // Funktionsaufruf Zeitdatenlogger
    }
    server.send(200, "application/json", "[\"" + static_cast<String>(relState[0] == ACTIVE) + "\",\"" + static_cast<String>(relState[1] == ACTIVE) + "\",\"" + localTime() + "\"]");
  });
}

void toSave() {
  File file = LittleFS.open(existFolder("Config") + "/dtime.dat", "w");
  if (file) {
    file.write(switchActive, sizeof(switchActive));
    file.write(wday, sizeof(wday));
    for (auto i = 0; i < COUNT * 2; i++) {
      file.printf("\n%s", switchTime[i]);
    }
    file.close();
  }
}

void dualTimerSwitch() {
  static uint8_t lastmin {60}, lastState[] {ACTIVE, ACTIVE};
  hobbsMeter(relState[0], relState[1]);                                // Funktionsaufruf Betriebsstundenzähler mit Relais Status
  if (tm.tm_min != lastmin) {
    lastmin = tm.tm_min;
    char buf[6];
    snprintf(buf, sizeof(buf), "%.2d:%.2d", tm.tm_hour, tm.tm_min);
    for (auto i = 0; i < COUNT * 2; i++) {
      if (switchActive[i / 2] && !strcmp(switchTime[i], buf)) {
        if (wday[i / 2] & (1 << (tm.tm_wday ? tm.tm_wday - 1 : 6))) {
          i < (COUNT % 2 ? COUNT + 1 : COUNT) ? ({relState[0] = i % 2 ? !ACTIVE : ACTIVE; timeDataLogger(0, "Programm");}) : ({relState[1] = i % 2 ? !ACTIVE : ACTIVE; timeDataLogger(1, "Programm");});  // Relais Status nach Zeit ändern
        }
      }
    }
  }
  if (relState[0] != lastState[0] || relState[1] != lastState[1]) {    // Relais schalten wenn sich der Status geändert hat
    for (auto i = 0; i < 2; i++) {
      lastState[i] = relState[i];
    }
    sonoffRelay_1();
    sonoffRelay_2();
  }
}
