// ****************************************************************
// Sketch Esp8266 Betriebsstundenzähler Dual Modular(Tab)
// created: Jens Fleischer, 2018-11-25
// last mod: Jens Fleischer, 2020-12-21
// For more information visit: https://fipsok.de
// ****************************************************************
// Hardware: Esp8266
// Software: Esp8266 Arduino Core 2.4.2 / 2.5.2 / 2.6.3 / 2.7.4
// Getestet auf: Nodemcu, Wemos D1 Mini Pro, Sonoff Dual
/******************************************************************
  Copyright (c) 2018 Jens Fleischer. All rights reserved.

  This file is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.
  This file is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.
*******************************************************************/
// Diese Version von Betriebsstundenzähler sollte als Tab eingebunden werden.
// #include <LittleFS.h> muss im Haupttab aufgerufen werden.
// Die Funktion "setupHobbsMeter();" muss im Setup aufgerufen werden.
// Der Admin Tab ist zum ausführen erforderlich.
/**************************************************************************************/

uint32_t totalmin[2], lastmin[2];

void setupHobbsMeter() {                                                   // Betriebstunden(minuten) beim Neustart einlesen
  File file = LittleFS.open(existFolder("Config") + "/hobbs.json", "r");
  if (file) {
    char buf[file.size()];
    file.readBytes(buf, sizeof buf);
    lastmin[0] = totalmin[0] = atoi(strtok(buf, "{\":runtimeOne"));
    lastmin[1] = totalmin[1] = atoi(strtok(NULL, "\":,runtimeTwo"));
    file.close();
  }
}

void hobbsMeter(bool &state_0, bool &state_1) {                            // Aufrufen mit Relais Status
  static uint32_t previousMillis[2];
  uint32_t currentMillis {millis()};
  if (currentMillis - previousMillis[0] >= 6e4) {
    previousMillis[0] = currentMillis;
    if (state_0 == ACTIVE) totalmin[0]++;                                  // Betriebstundenzähler Relais 1 wird um eine Minute erhöht
    if (state_1 == ACTIVE) totalmin[1]++;                                  // Betriebstundenzähler Relais 2 wird um eine Minute erhöht
  }
  if (currentMillis - previousMillis[1] >= 1728e5 && (totalmin[0] != lastmin[0] || totalmin[1] != lastmin[1])) {  // aller 2 Tage Betriebsstunden in Datei schreiben
    previousMillis[1] = currentMillis;
    lastmin[0] = totalmin[0];
    lastmin[1] = totalmin[1];
    save();
  }
}

String extra() {                                                           // Betriebsstundenanzeige admin.html
  return "\", \"HourMeter1\":\"" + operatingTime(totalmin[0]) + "\", \"HourMeter2\":\"" +  operatingTime(totalmin[1]);
}

String operatingTime(uint32_t &tmin) {                                     // lesbare Anzeige der Betriebstunden(minuten)
  char buf[9];
  snprintf(buf, sizeof(buf), "%d,%d", tmin / 60, tmin / 6 % 10);
  return buf;
}

void save() {
  File file = LittleFS.open(existFolder("Config") + "/hobbs.json", "w");   // Betriebstunden(minuten) speichern
  if (file) {
    file.printf(R"({"runtimeOne":"%u","runtimeTwo":"%u"})", totalmin[0], totalmin[1]);
    file.close();
  }
}
